import os
import json
import singer
from tap_clubspeed.streams import STREAMS


def get_abs_path(path):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), path)


def discover_streams(client):
    streams = []

    for s in STREAMS.values():
        s = s(client)
        schema = singer.resolve_schema_references(s.load_schema())
        streams.append({'stream': s.name, 'tap_stream_id': s.name, 'schema': schema, 'metadata': s.load_metadata()})
    return streams







#!/usr/bin/env python3
import json
import sys
import singer
from singer import metadata
from tap_clubspeed.clubspeed import Clubspeed
from tap_clubspeed.discover import discover_streams
from tap_clubspeed.sync import sync_stream
from tap_clubspeed.streams import STREAMS

LOGGER = singer.get_logger()

REQUIRED_CONFIG_KEYS = [
    "subdomain",
    "private_key"
]


def do_discover(client):
    LOGGER.info("Starting discover")
    catalog = {"streams": discover_streams(client)}
    json.dump(catalog, sys.stdout, indent=2)
    LOGGER.info("Finished discover")


def stream_is_selected(mdata):
    return mdata.get((), {}).get('selected', False)


def get_selected_streams(catalog):
    selected_stream_names = []
    for stream in catalog.streams:
        mdata = metadata.to_map(stream.metadata)
        if stream_is_selected(mdata):
            selected_stream_names.append(stream.tap_stream_id)
    return selected_stream_names


class DependencyException(Exception):
    pass


def populate_class_schemas(catalog, selected_stream_names):
    for stream in catalog.streams:
        if stream.tap_stream_id in selected_stream_names:
            STREAMS[stream.tap_stream_id].stream = stream


def ensure_credentials_are_authorized(client):
    client.is_authorized()


def do_sync(client, catalog, state):
    ensure_credentials_are_authorized(client)
    selected_stream_names = get_selected_streams(catalog)
    populate_class_schemas(catalog, selected_stream_names)

    for stream in catalog.streams:
        stream_name = stream.tap_stream_id

        mdata = metadata.to_map(stream.metadata)

        if stream_name not in selected_stream_names:
            LOGGER.info("%s: Skipping - not selected", stream_name)
            continue

        key_properties = metadata.get(mdata, (), 'table-key-properties')
        singer.write_schema(stream_name, stream.schema.to_dict(), key_properties)

        LOGGER.info("%s: Starting sync", stream_name)
        instance = STREAMS[stream_name](client)
        instance.stream = stream
        counter_value = sync_stream(state, instance)
        singer.write_state(state)
        LOGGER.info("%s: Completed sync (%s rows)", stream_name, counter_value)

    singer.write_state(state)
    LOGGER.info("Finished sync")


@singer.utils.handle_top_exception(LOGGER)
def main():
    parsed_args = singer.utils.parse_args(REQUIRED_CONFIG_KEYS)

    creds = {
        "subdomain": parsed_args.config['subdomain'],
        "private_key": parsed_args.config['private_key']
    }
    client = Clubspeed(**creds)

    if parsed_args.discover:
        do_discover(client)
    elif parsed_args.catalog:
        state = parsed_args.state or {}
        do_sync(client, parsed_args.catalog, state)




import requests
import logging

logger = logging.getLogger()


class IgnoreHttpException(Exception):
    pass


class Clubspeed(object):


    def __init__(self, subdomain=None, private_key=None, session=None):
        """ Simple Python wrapper for the Clubspeed API. Only supports GET. """
        self.protocol = 'https'
        self.domain = 'clubspeedtiming.com'
        self.subdomain = subdomain
        self.api_prefix = 'api/index.php/'
        self.private_key = private_key
        self._new_heats = []
        self._url_template = "{protocol}://{subdomain}.{domain}/{api_prefix}{path}.json?key={private_key}"
        self._limit = 100
        self._test = False


    def _get(self, url, **kwargs):
        logger.info("Hitting endpoint {url}".format(url=url))
        response = requests.get(url)
        if response.status_code == 500:
            raise IgnoreHttpException("http status is 500.")
        response.raise_for_status()
        return response.json()


    def _construct_endpoint(self, path):
        return self._url_template.format(protocol=self.protocol, 
                                         subdomain=self.subdomain, 
                                         domain=self.domain, 
                                         api_prefix=self.api_prefix,
                                         path=path,
                                         private_key=self.private_key)


    def _set_page_in_endpoint(self, endpoint, page=0):
        if "&page=" not in endpoint:
            endpoint += "&page=0&limit={limit}".format(limit=self._limit)
        else:
            array = endpoint.split('&')
            index = 0
            while index < len(array):
                if "page=" in array[index]:
                    array[index] = "page=" + str(page)
                index += 1
            endpoint = '&'.join(array)
        return endpoint


    def _add_filter(self, endpoint, api_version, column_name, bookmark):
        if column_name is None:
            return endpoint
        if bookmark is None:
            if api_version == 'V2':
                endpoint += '&where={{"{column_name}":{{"$isnot":"null"}}}}&order={column_name} ASC'.format(column_name=column_name)
            else:
                endpoint += '&filter={column_name} IS NOT NULL&order={column_name} ASC'.format(column_name=column_name)
            return endpoint
        if api_version == 'V2':
            endpoint += '&where={{"{column_name}":{{"$gt":"{bookmark}"}}}}&order={column_name} ASC'.format(column_name=column_name, bookmark=bookmark)
        else:
            endpoint += '&filter={column_name} > {bookmark}&order={column_name} ASC'.format(column_name=column_name, bookmark=bookmark)
        return endpoint


    def _get_response(self, endpoint, key=None):
        length = 1
        page = 0
        while length > 0:
            endpoint = self._set_page_in_endpoint(endpoint, page)
            try:
                res = self._get(endpoint)
                res = res[key] if key is not None else res
                length = len(res)
                logger.info('Endpoint returned {length} rows.'.format(length=length))
                for item in res:
                    if 'heatMain' in endpoint and 'heatId' in item:
                        self._new_heats.append(item['heatId'])
                    yield item
            except IgnoreHttpException:
                logger.info('Encountered 500, will ignore.')
                pass
            if self._test and page >= 2:
                break
            page += 1


    def is_authorized(self):
        endpoint = self._construct_endpoint('payments')
        return self._get(endpoint)


    def booking(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('booking')
        endpoint = self._add_filter(endpoint, 'V1', column_name, bookmark)
        return self._get_response(endpoint, 'bookings')


    def booking_availability(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('bookingAvailability')
        endpoint = self._add_filter(endpoint, 'V1', column_name, bookmark)
        return self._get_response(endpoint, 'bookings')


    def check_details(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('checkDetails')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint, 'checkDetails')


    def checks(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('checks')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint, 'checks')


    def check_totals(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('checkTotals')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def customers(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('customers')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def discount_types(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('discountType')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_heat_details(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventHeatDetails')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_heat_types(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventHeatTypes')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_reservation_links(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventReservationLinks')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_reservations(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventReservations')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_reservation_types(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventReservationTypes')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_rounds(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventRounds')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def events(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('events')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_statuses(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventStatuses')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_tasks(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventTasks')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_task_types(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventTaskTypes')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def event_types(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('eventTypes')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def gift_card_history(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('giftCardHistory')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    # Note: This function pulls `heat_details` from the API, but since
    # the API doesn't have a `last_edited_at` field, we use `heat_main.finish`
    # to determine when to pull `heat_details`.
    def heat_main_details(self, column_name=None, bookmark=None):
        endpoints = []
        idx = 0
        query = ''
        endpoint_base = self._construct_endpoint('heatDetails')

        # Using `heat_id`s, create array of endpoints for `heat_details`
        for heat_id in self._new_heats:
            query += '{{"heatId":{heat_id}}}'.format(heat_id=heat_id)
            if (idx is not 0 and idx % 30 == 0) or idx == len(self._new_heats) - 1:
                endpoint = endpoint_base + '&where={{"$or":[{query}]}}&order={column_name} ASC'.format(query=query, column_name='heatId')
                endpoints.append(endpoint)
                query = ''
            else:
                query += ','
            idx += 1

        # Yield all results from endpoints.
        for endpoint in endpoints:
            gtr = self._get_response(endpoint)
            for item in gtr:
                yield item


    def heat_main(self, column_name=None, bookmark=None):
        self._new_heats = []
        endpoint = self._construct_endpoint('heatMain')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def heat_types(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('heatTypes')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def memberships(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('memberships')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def membership_types(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('membershipTypes')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def payments(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('payments')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def payments_voided(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('payments')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def product_classes(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('productClasses')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def products(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('products')
        endpoint = self._add_filter(endpoint, 'V1', column_name, bookmark)
        return self._get_response(endpoint, 'products')


    def reservations(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('reservations')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint, 'reservations')


    def sources(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('sources')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)


    def taxes(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('taxes')
        endpoint = self._add_filter(endpoint, 'V1', column_name, bookmark)
        return self._get_response(endpoint, 'taxes')


    def users(self, column_name=None, bookmark=None):
        endpoint = self._construct_endpoint('users')
        endpoint = self._add_filter(endpoint, 'V2', column_name, bookmark)
        return self._get_response(endpoint)








import os
import json
import datetime
import pytz
import singer
from singer import metadata
from singer import utils
from singer.metrics import Point
from dateutil.parser import parse


logger = singer.get_logger()
KEY_PROPERTIES = ['id']


def get_abs_path(path):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), path)


def needs_parse_to_date(string):
    if isinstance(string, str):
        try: 
            parse(string)
            return True
        except ValueError:
            return False
    return False


class Stream():
    name = None
    replication_method = None
    replication_key = None
    stream = None
    key_properties = KEY_PROPERTIES


    def __init__(self, client=None):
        self.client = client


    def get_bookmark(self, state):
        return singer.get_bookmark(state, self.name, self.replication_key)


    def update_bookmark(self, state, value):
        current_bookmark = self.get_bookmark(state)
        if value and needs_parse_to_date(value) and needs_parse_to_date(current_bookmark):
            if utils.strptime_with_tz(value) > utils.strptime_with_tz(current_bookmark):
                singer.write_bookmark(state, self.name, self.replication_key, value)
        elif current_bookmark is None:
            singer.write_bookmark(state, self.name, self.replication_key, value)


    # This function returns boolean and checks if
    # book mark is old.
    def is_bookmark_old(self, state, value):
        current_bookmark = self.get_bookmark(state)
        if current_bookmark is None:
            return True
        if needs_parse_to_date(current_bookmark) and needs_parse_to_date(value):
            if utils.strptime_with_tz(value) >= utils.strptime_with_tz(current_bookmark):
                return True
        else:
            if int(value) >= int(current_bookmark):
                return True
        return False


    def load_schema(self):
        schema_file = "schemas/{}.json".format(self.name)
        with open(get_abs_path(schema_file)) as f:
            schema = json.load(f)
        return self._add_custom_fields(schema)


    def _add_custom_fields(self, schema): # pylint: disable=no-self-use
        return schema


    def load_metadata(self):
        schema = self.load_schema()
        mdata = metadata.new()

        mdata = metadata.write(mdata, (), 'table-key-properties', self.key_properties)
        mdata = metadata.write(mdata, (), 'forced-replication-method', self.replication_method)

        if self.replication_key:
            mdata = metadata.write(mdata, (), 'valid-replication-keys', [self.replication_key])

        for field_name in schema['properties'].keys():
            if field_name in self.key_properties or field_name == self.replication_key:
                mdata = metadata.write(mdata, ('properties', field_name), 'inclusion', 'automatic')
            else:
                mdata = metadata.write(mdata, ('properties', field_name), 'inclusion', 'available')

        return metadata.to_list(mdata)


    def is_selected(self):
        return self.stream is not None


    # The main sync function.
    def sync(self, state):
        get_data = getattr(self.client, self.name)

        bookmark = self.get_bookmark(state)

        res = get_data(self.replication_key, bookmark)

        if self.replication_method == "INCREMENTAL":
            for item in res:
                try:
                    if self.is_bookmark_old(state, item[self.replication_key]):
                        self.update_bookmark(state, item[self.replication_key])
                        yield (self.stream, item)

                except KeyError:
                    logger.info('Bookmark doesn\'t exist: syncing row.')
                    yield (self.stream, item)
                    pass

                except Exception as e:
                    logger.error('Handled exception: {error}'.format(error=str(e)))
                    pass

        elif self.replication_method == "FULL_TABLE":
            for item in res:
                yield (self.stream, item)

        else:
            raise Exception('Replication key not defined for {stream}'.format(self.name))



class Booking(Stream):
    name = "booking"
    replication_method = "FULL_TABLE"
    key_properties = [ "onlineBookingsId" ]


class BookingAvailability(Stream):
    name = "booking_availability"
    replication_method = "FULL_TABLE"
    key_properties = ["heatId"]


class CheckDetails(Stream):
    name = "check_details"
    replication_method = "INCREMENTAL"
    replication_key = "createdDate"
    key_properties = [ "checkDetailId" ]


class Checks(Stream):
    name = "checks"
    replication_method = "INCREMENTAL"
    replication_key = "closedDate"
    key_properties = [ "checkId" ]


class CheckTotals(Stream):
    name = "check_totals"
    replication_method = "INCREMENTAL"
    replication_key = "closedDate"
    key_properties = [ "checkId" ]


class Customers(Stream):
    name = "customers"
    replication_method = "INCREMENTAL"
    replication_key = "lastVisited"
    key_properties = [ "customerId" ]


class DiscountTypes(Stream):
    name = "discount_types"
    replication_method = "FULL_TABLE"
    key_properties = ["discountId"]


class EventHeatDetails(Stream):
    name = "event_heat_details"
    replication_method = "INCREMENTAL"
    replication_key = "added" 
    key_properties = [ "eventId" ]


class EventHeatTypes(Stream):
    name = "event_heat_types"
    replication_method = "FULL_TABLE"
    key_properties = ["eventHeatTypeId"]


class EventReservationLinks(Stream): 
    name = "event_reservation_links"
    replication_method = "FULL_TABLE"
    key_properties = ["eventReservationLinkId"]


class EventReservations(Stream):
    name = "event_reservations"
    replication_method = "INCREMENTAL"
    replication_key = "startTime"
    key_properties = [ "eventReservationId" ]


class EventReservationTypes(Stream):
    name = "event_reservation_types"
    replication_method = "FULL_TABLE"
    key_properties = ["eventReservationTypeId"]


class EventRounds(Stream):
    name = "event_rounds"
    replication_method = "FULL_TABLE"
    key_properties = ["eventRoundId"]


class Events(Stream):
    name = "events"
    replication_method = "INCREMENTAL"
    replication_key = "eventScheduledTime"
    key_properties = [ "eventId" ]


class EventStatuses(Stream):
    name = "event_statuses"
    replication_method = "FULL_TABLE"
    key_properties = ["eventStatusId"]


class EventTasks(Stream):
    name = "event_tasks"
    replication_method = "INCREMENTAL"
    replication_key = "completedAt"
    key_properties = [ "eventTaskId" ]


class EventTaskTypes(Stream):
    name = "event_task_types"
    replication_method = "FULL_TABLE"
    key_properties = [ "eventTaskId" ]


class EventTypes(Stream):
    name = "event_types"
    replication_method = "FULL_TABLE"
    key_properties = [ "eventTypeId" ]


class GiftCardHistory(Stream):
    name = "gift_card_history"
    replication_method = "INCREMENTAL"
    replication_key = "transactionDate"
    key_properties = [ "giftCardHistoryId" ]


class HeatMain(Stream):
    name = "heat_main"
    replication_method = "INCREMENTAL"
    replication_key = "finish"
    key_properties = [ "heatId" ]


#
# This table uses heat_main's bookmark as it's bookmark,
# but to allow this to work, we give the replication_key
# an intentional "None".
#

class HeatMainDetails(Stream):
    name = "heat_main_details"
    replication_method = "INCREMENTAL"
    replication_key = "None"
    key_properties = [ "heatId" ]


class HeatTypes(Stream):
    name = "heat_types"
    replication_method = "FULL_TABLE"
    key_properties = ["heatTypesId"]


class Memberships(Stream):
    name = "memberships"
    replication_method = "INCREMENTAL"
    replication_key = "changed"
    key_properties = [ "membershipTypeId" ]


class MembershipTypes(Stream):
    name = "membership_types"
    replication_method = "FULL_TABLE"
    key_properties = ["membershipTypeId"]


class Payments(Stream):
    name = "payments"
    replication_method = "INCREMENTAL"
    replication_key = "payDate"
    key_properties = [ "paymentId" ]


class PaymentsVoided(Stream):
    name = "payments_voided"
    replication_method = "INCREMENTAL"
    replication_key = "voidDate"
    key_properties = [ "paymentId" ]


class ProductClasses(Stream):
    name = "product_classes"
    replication_method = "FULL_TABLE"
    key_properties = ["productClassId"]


class Products(Stream):
    name = "products"
    replication_method = "FULL_TABLE"
    key_properties = ["productId"]


class Reservations(Stream):
    name = "reservations"
    replication_method = "INCREMENTAL"
    replication_key = "createdAt"
    key_properties = [ "onlineBookingReservationsId" ]


class Sources(Stream):
    name = "sources"
    replication_method = "FULL_TABLE"
    key_properties = ["sourceId"]


class Taxes(Stream):
    name = "taxes"
    replication_method = "FULL_TABLE"
    key_properties = ["taxId"]


class Users(Stream):
    name = "users"
    replication_method = "FULL_TABLE"
    key_properties = ["userId"]



STREAMS = {
    "booking": Booking,
    "booking_availability": BookingAvailability,
    "check_details": CheckDetails,
    "checks": Checks,
    "customers": Customers,
    "discount_types": DiscountTypes,
    "event_heat_details": EventHeatDetails,
    "event_heat_types": EventHeatTypes,
    "event_reservation_links": EventReservationLinks,
    "event_reservations": EventReservations,
    "event_reservation_types": EventReservationTypes,
    "event_rounds": EventRounds,
    "events": Events,
    "event_statuses": EventStatuses,
    "event_tasks": EventTasks,
    "event_task_types": EventTaskTypes,
    "event_types": EventTypes,
    "gift_card_history": GiftCardHistory,
    "heat_main": HeatMain,
    "heat_main_details": HeatMainDetails,
    "heat_types": HeatTypes,
    "memberships": Memberships,
    "membership_types": MembershipTypes,
    "payments": Payments,
    "payments_voided": PaymentsVoided,
    "product_classes": ProductClasses,
    "products": Products,
    "reservations": Reservations,
    "sources": Sources,
    "taxes": Taxes,
    "users": Users
}



#!/usr/bin/env python

from setuptools import setup

setup(name='tap-clubspeed',
      version='1.0.0',
      description='Singer.io tap for extracting data from the Clubspeed API',
      author='lambtron',
      author_email="andyjiang@gmail.com",
      url='https://www.singer.io/',
      classifiers=['Programming Language :: Python :: 3 :: Only'],
      py_modules=['tap_clubspeed'],
      install_requires=[
          'singer-python==5.1.5',
          'requests==2.20.0'
      ],
      entry_points='''
          [console_scripts]
          tap-clubspeed=tap_clubspeed:main
      ''',
      packages=['tap_clubspeed'],
      include_package_data=True,
)
